@@ -188,7 +188,7 @@ struct propa_ptr props_serial[] = {
 	PROPA_PTR_END("name", "a:", 3),
 };
 
-unsigned long amba_systemid; /* Set by oprom_init code */
+unsigned long amba_systemid; /* Set by ambapp_init */
 unsigned long amba_ioarea = CONFIG_AMBA_IO_AREA;
 unsigned long ipi_num = CONFIG_IPI_NUM;
 
@@ -295,7 +295,10 @@ void *prom_malloc(int size)
 {
 	size = (size + 3) & ~3;
 	if ((prom_malloc_end - size) < (unsigned long)&_prom_heap_start) {
-		/* ERROR. RESIZE PROM HEAP WITH CONFIG_PROM_HEAP_SIZE */
+		/*
+		 * ERROR: Resize prom heap size with -promheap option that
+		 * sets up CONFIG_PROM_HEAP_SIZE.
+		 */
 		while (1)
 			;
 	}
@@ -483,10 +486,9 @@ void opprom_init(void)
 		ncpu = CONFIG_MAX_CPUS;
 #endif
 
+#if (CONFIG_WAKE_CPU_MASK != 0)
 	/* Wake other (disabled) CPUs if user has requested it */
-#define CPU_WAKE_MASK ((0xff << CONFIG_MAX_CPUS) & 0xff)
-#if (CONFIG_WAKE_DISABLED_CPUS > 0) && (CPU_WAKE_MASK != 0)
-	BYPASS_STORE_PA(&irqmp->mpstatus, CPU_WAKE_MASK);
+	BYPASS_STORE_PA(&irqmp->mpstatus, CONFIG_WAKE_CPU_MASK);
 #endif
 	nodes_lvl2[NODE_CPU1_IDX - 1 + (ncpu - 1)].sibling = NULL;
 
@@ -496,10 +498,6 @@ void opprom_init(void)
 	nodes_lvl2[NODE_AMBAPP_IDX].child = pnodes_amba;
 #endif
 
-	/* Get AMBA System ID */
-	amba_systemid =
-		BYPASS_LOAD_PA(CONFIG_AMBA_IO_AREA | AMBA_CONF_AREA | 0xff0);
-
 	opprom_setup_banner_name();
 
 	/* Calculate IDPROM Checksum */
