@@ -11,6 +11,10 @@
 #include <apbuart.h>
 #include <iommu.h>
 
+#define GR712RC_SYSTEMID 0x7120e70
+#define GR740_REV0_SYSTEMID 0x07401038
+#define GR740_REV1_SYSTEMID 0x07401039
+
 #ifdef CONFIG_AMPOPTS
 struct amp_opt {
 	short idx, val;
@@ -493,8 +497,9 @@ int grgpio_fixup_core(ambapp_core *core)
 	ambapp_apbdev *dev;
 	int irqgen;
 
+	/* Static solution as core subindex is not generally available */
 	static unsigned int totalcores = 0;
-	int idx = totalcores++; /* core->subindex not initialized this early */
+	int idx = totalcores++;
 
 	dev = core->apbslv;
 	regs = (struct grgpio_regs *)dev->address;
@@ -606,9 +611,34 @@ void greth_init(ambapp_core *core, int level)
 	}
 }
 
-#define GR712RC_SYSTEMID 0x7120e70
-#define GR740_REV0_SYSTEMID 0x07401038
-#define GR740_REV1_SYSTEMID 0x07401039
+int greth_fixup_prop(ambapp_core *core, struct prop_std *buf, int index)
+{
+	switch (index) {
+	case 0:
+		/* Must be last, as we fallthrough to signal no more props */
+		if ((amba_systemid == GR740_REV0_SYSTEMID) ||
+		    (amba_systemid == GR740_REV1_SYSTEMID)) {
+			/*
+			 * The two greth cores shares MDIO bus but greth0 is
+			 * nevertheless supposed to only use the PHY with addr 1
+			 * and greth1 the PHY with addr 2.
+			 *
+			 * Use static numgreths as core subindex is not
+			 * generally available.
+			 */
+			static char *prop_str_forcephy;
+			static int numgreths = 0;
+			int subindex = numgreths++;
+			int phyaddr = subindex + 1;
+
+			PROM_STRDUP_ONCE(prop_str_forcephy, "fixed-phy-addr");
+			return ambapp_prop_int(buf, prop_str_forcephy, phyaddr);
+		}
+		/* Fallthrough */
+	default:
+		return 0;
+	}
+}
 
 void canmux_init(ambapp_core *core, int level)
 {
@@ -616,10 +646,7 @@ void canmux_init(ambapp_core *core, int level)
 		uint32_t ctrl;
 	} *regs;
 
-	unsigned long systemid =
-		BYPASS_LOAD_PA(CONFIG_AMBA_IO_AREA | AMBA_CONF_AREA | 0xff0);
-
-	if (level == 0 && core->apbslv && systemid == GR712RC_SYSTEMID) {
+	if (level == 0 && core->apbslv && amba_systemid == GR712RC_SYSTEMID) {
 		regs = (struct canmux_regs *)core->apbslv->address;
 
 		/* Use both CAN_OC cores */
@@ -676,7 +703,7 @@ struct amba_driver amba_drivers[] = {
  AMBAPP_DRV(VENDOR_GAISLER, GAISLER_LEON3, NULL, NULL, NULL, leon_init),
  AMBAPP_DRV(VENDOR_GAISLER, GAISLER_LEON3FT, NULL, NULL, NULL, leon_init),
  AMBAPP_DRV(VENDOR_GAISLER, GAISLER_APBUART, apbuart_apb_fixup, NULL, NULL, NULL),
- AMBAPP_DRV(VENDOR_GAISLER, GAISLER_ETHMAC, NULL, NULL, NULL, greth_init),
+ AMBAPP_DRV(VENDOR_GAISLER, GAISLER_ETHMAC, NULL, NULL, greth_fixup_prop, greth_init),
  AMBAPP_DRV(VENDOR_GAISLER, GAISLER_I2CMST, NULL, NULL, i2c_fixup_prop, NULL),
  AMBAPP_DRV(VENDOR_GAISLER, GAISLER_SPICTRL, NULL, NULL, spi_fixup_prop, NULL),
  AMBAPP_DRV(VENDOR_GAISLER, GAISLER_GPIO, NULL, grgpio_fixup_core, grgpio_fixup_prop, NULL),
@@ -929,7 +956,6 @@ struct ambapp_bridge_driver {
 };
 
 /* proc.c */
-extern unsigned long amba_systemid;
 extern unsigned long ipi_num;
 struct propa_ptr props_ambapp_templ[] = {
 	PROPA_PTR("device_type", "ambapp", 7),
@@ -978,16 +1004,13 @@ static int ahb2ahb_bus_create_iommu_fixup(
 	struct ambapp_bus *ainfo, struct ahb_bus *parent, ambapp_ahbdev *bridge,
 	struct ambapp_pnp_ahb *ahb, int index)
 {
-	unsigned long systemid =
-		BYPASS_LOAD_PA(CONFIG_AMBA_IO_AREA | AMBA_CONF_AREA | 0xff0);
-
-	if (((systemid == GR740_REV0_SYSTEMID) ||
-	     (systemid == GR740_REV1_SYSTEMID)) &&
+	if (((amba_systemid == GR740_REV0_SYSTEMID) ||
+	     (amba_systemid == GR740_REV1_SYSTEMID)) &&
 	    (index <= 9)) {
 		int pciver;
 		int spwver = 1;
 
-		if (systemid == GR740_REV1_SYSTEMID) {
+		if (amba_systemid == GR740_REV1_SYSTEMID) {
 			pciver = 2;
 		} else {
 			pciver = 1;
@@ -1752,6 +1775,10 @@ struct ambapp_bus ambapp_plb;
 /* Initialize AMBA Structures but not bus frequency, pair devices into cores */
 void ambapp_init(void)
 {
+	/* Get AMBA System ID */
+	amba_systemid =
+		BYPASS_LOAD_PA(CONFIG_AMBA_IO_AREA | AMBA_CONF_AREA | 0xff0);
+
 	ambapp_bus_init(&ambapp_plb, CONFIG_AMBA_IO_AREA);
 
 	ambapp_cores_init(&ambapp_plb);
@@ -2211,8 +2238,14 @@ void xml_nodes_init_hook(struct ambapp_bus *ainfo)
 	ambapp_core *core = ainfo->cores;
 	ambapp_core *cn;
 
-	/* Initiate subindex for cores with the same vendor and device by
-	 * looping through all cores and for each core with increasing */
+       /*
+	* Initiate subindex for cores with the same vendor and device by
+	* looping through all cores and for each core increasing subindex of
+	* all cores behind it of the same type.
+	*
+	* E.g. the third found uart has its subindex increased once by the
+	* first, and once by the second, making its subindex 2 as intended.
+	*/
 	while (core) {
 		ambapp_dev *a = ambapp_get_first_dev(core);
 		cn = core->next;
@@ -2220,7 +2253,7 @@ void xml_nodes_init_hook(struct ambapp_bus *ainfo)
 			ambapp_dev *b = ambapp_get_first_dev(cn);
 			if (a->device == b->device &&
 			    a->vendor == b->vendor) {
-				cn->subindex++;
+				cn->subindex_for_xml++;
 			}
 			cn = cn->next;
 		}
@@ -2283,7 +2316,7 @@ int xml_core_matches(ambapp_core *core, struct xml_match *m)
 	if (m->vendor && m->device) {
 		/* Match by vendor, device, core subindex */
 		if (m->vendor == a->vendor && m->device == a->device &&
-		    (m->index == -1 || m->index == core->subindex)) {
+		    (m->index == -1 || m->index == core->subindex_for_xml)) {
 			return 1;
 		}
 	} else if (core->index == m->coreindex) {
